@@ -10,12 +10,13 @@ import (
 	"net"
 	"net/http"
 
+	"github.com/samber/lo"
+
 	"github.com/superfly/flyctl/agent"
 	"github.com/superfly/flyctl/api"
+	"github.com/superfly/flyctl/client"
 	"github.com/superfly/flyctl/flyctl"
 	"github.com/superfly/flyctl/internal/logger"
-
-	"github.com/superfly/flyctl/client"
 )
 
 var NonceHeader = "fly-machine-lease-nonce"
@@ -28,7 +29,6 @@ type Client struct {
 }
 
 func New(ctx context.Context, app *api.AppCompact) (*Client, error) {
-
 	logger := logger.MaybeFromContext(ctx)
 
 	client := client.FromContext(ctx).API()
@@ -62,7 +62,7 @@ func New(ctx context.Context, app *api.AppCompact) (*Client, error) {
 }
 
 func (f *Client) CreateApp(ctx context.Context, name string, org string) (err error) {
-	var in = map[string]interface{}{
+	in := map[string]interface{}{
 		"app_name": name,
 		"org_slug": org,
 	}
@@ -77,7 +77,7 @@ func (f *Client) Launch(ctx context.Context, builder api.LaunchMachineInput) (*a
 		endpoint = fmt.Sprintf("/%s", builder.ID)
 	}
 
-	var out = new(api.Machine)
+	out := new(api.Machine)
 
 	if err := f.sendRequest(ctx, http.MethodPost, endpoint, builder, out, nil); err != nil {
 		return nil, fmt.Errorf("failed to launch VM: %w", err)
@@ -87,7 +87,7 @@ func (f *Client) Launch(ctx context.Context, builder api.LaunchMachineInput) (*a
 }
 
 func (f *Client) Update(ctx context.Context, builder api.LaunchMachineInput, nonce string) (*api.Machine, error) {
-	var headers = make(map[string][]string)
+	headers := make(map[string][]string)
 
 	if nonce != "" {
 		headers[NonceHeader] = []string{nonce}
@@ -95,7 +95,7 @@ func (f *Client) Update(ctx context.Context, builder api.LaunchMachineInput, non
 
 	endpoint := fmt.Sprintf("/%s", builder.ID)
 
-	var out = new(api.Machine)
+	out := new(api.Machine)
 
 	if err := f.sendRequest(ctx, http.MethodPost, endpoint, builder, out, headers); err != nil {
 		return nil, fmt.Errorf("failed to update VM %s: %w", builder.ID, err)
@@ -114,7 +114,7 @@ func (f *Client) Start(ctx context.Context, machineID string) (*api.MachineStart
 	return out, nil
 }
 
-func (f *Client) Wait(ctx context.Context, machine *api.Machine) (err error) {
+func (f *Client) Wait(ctx context.Context, machine *api.Machine, state string) (err error) {
 	waitEndpoint := fmt.Sprintf("/%s/wait", machine.ID)
 
 	version := machine.InstanceID
@@ -128,8 +128,14 @@ func (f *Client) Wait(ctx context.Context, machine *api.Machine) (err error) {
 		waitEndpoint += "?timeout=30"
 	}
 
+	if state == "" {
+		state = "started"
+	}
+
+	waitEndpoint += fmt.Sprintf("&state=%s", state)
+
 	if err := f.sendRequest(ctx, http.MethodGet, waitEndpoint, nil, nil, nil); err != nil {
-		return fmt.Errorf("failed to wait for VM %s: %w", machine.ID, err)
+		return fmt.Errorf("failed to wait for VM %s in %s state: %w", machine.ID, state, err)
 	}
 	return
 }
@@ -144,7 +150,7 @@ func (f *Client) Stop(ctx context.Context, machine api.StopMachineInput) (err er
 }
 
 func (f *Client) Get(ctx context.Context, machineID string) (*api.Machine, error) {
-	var getEndpoint = ""
+	getEndpoint := ""
 
 	if machineID != "" {
 		getEndpoint = fmt.Sprintf("/%s", machineID)
@@ -160,7 +166,7 @@ func (f *Client) Get(ctx context.Context, machineID string) (*api.Machine, error
 }
 
 func (f *Client) List(ctx context.Context, state string) ([]*api.Machine, error) {
-	var getEndpoint = ""
+	getEndpoint := ""
 
 	if state != "" {
 		getEndpoint = fmt.Sprintf("?%s", state)
@@ -175,6 +181,24 @@ func (f *Client) List(ctx context.Context, state string) ([]*api.Machine, error)
 	return out, nil
 }
 
+// ListActive returns only non-destroyed that aren't in a reserved process group.
+func (f *Client) ListActive(ctx context.Context) ([]*api.Machine, error) {
+	getEndpoint := ""
+
+	machines := make([]*api.Machine, 0)
+
+	err := f.sendRequest(ctx, http.MethodGet, getEndpoint, nil, &machines, nil)
+	if err != nil {
+		return nil, fmt.Errorf("failed to list VMs: %w", err)
+	}
+
+	machines = lo.Filter(machines, func(m *api.Machine, _ int) bool {
+		return m.Config.Metadata["process_group"] != "release_command" && m.State != "destroyed"
+	})
+
+	return machines, nil
+}
+
 func (f *Client) Destroy(ctx context.Context, input api.RemoveMachineInput) (err error) {
 	destroyEndpoint := fmt.Sprintf("/%s?kill=%t", input.ID, input.Kill)
 
@@ -186,8 +210,7 @@ func (f *Client) Destroy(ctx context.Context, input api.RemoveMachineInput) (err
 }
 
 func (f *Client) Kill(ctx context.Context, machineID string) (err error) {
-
-	var in = map[string]interface{}{
+	in := map[string]interface{}{
 		"signal": 9,
 	}
 	err = f.sendRequest(ctx, http.MethodPost, fmt.Sprintf("/%s/signal", machineID), in, nil, nil)
@@ -199,7 +222,7 @@ func (f *Client) Kill(ctx context.Context, machineID string) (err error) {
 }
 
 func (f *Client) GetLease(ctx context.Context, machineID string, ttl *int) (*api.MachineLease, error) {
-	var endpoint = fmt.Sprintf("/%s/lease", machineID)
+	endpoint := fmt.Sprintf("/%s/lease", machineID)
 
 	if ttl != nil {
 		endpoint += fmt.Sprintf("?ttl=%d", *ttl)
@@ -215,9 +238,9 @@ func (f *Client) GetLease(ctx context.Context, machineID string, ttl *int) (*api
 }
 
 func (f *Client) ReleaseLease(ctx context.Context, machineID, nonce string) error {
-	var endpoint = fmt.Sprintf("/%s/lease", machineID)
+	endpoint := fmt.Sprintf("/%s/lease", machineID)
 
-	var headers = make(map[string][]string)
+	headers := make(map[string][]string)
 
 	if nonce != "" {
 		headers[NonceHeader] = []string{nonce}
@@ -227,7 +250,6 @@ func (f *Client) ReleaseLease(ctx context.Context, machineID, nonce string) erro
 }
 
 func (f *Client) sendRequest(ctx context.Context, method, endpoint string, in, out interface{}, headers map[string][]string) error {
-
 	req, err := f.NewRequest(ctx, method, endpoint, in, headers)
 	if err != nil {
 		return err
@@ -268,7 +290,6 @@ func (f *Client) NewRequest(ctx context.Context, method, path string, in interfa
 			return nil, err
 		}
 		headers["Content-Type"] = []string{"application/json"}
-
 		body = bytes.NewReader(b)
 	}
 
