@@ -236,19 +236,17 @@ func DeployMachinesApp(ctx context.Context, app *api.AppCompact, strategy string
 	if len(machines) > 0 {
 
 		for _, machine := range machines {
-
 			leaseTTL := api.IntPointer(30)
 			lease, err := flapsClient.GetLease(ctx, machine.ID, leaseTTL)
 			if err != nil {
 				return err
 			}
-
 			machine.LeaseNonce = lease.Data.Nonce
 
+			defer releaseLease(ctx, flapsClient, machine)
 		}
 
 		for _, machine := range machines {
-
 			launchInput.ID = machine.ID
 
 			// We assume a config with no image specificed means the deploy should recreate machines
@@ -273,12 +271,6 @@ func DeployMachinesApp(ctx context.Context, app *api.AppCompact, strategy string
 			updateResult, err := flapsClient.Update(ctx, launchInput, machine.LeaseNonce)
 			if err != nil {
 				if strategy != "immediate" {
-					leaseErr := flapsClient.ReleaseLease(ctx, machine.ID, machine.LeaseNonce)
-
-					if leaseErr != nil {
-						fmt.Fprintf(io.Out, "Could not release lease for %s\n", machine.ID)
-					}
-
 					return err
 
 				} else {
@@ -288,28 +280,15 @@ func DeployMachinesApp(ctx context.Context, app *api.AppCompact, strategy string
 
 			if strategy != "immediate" {
 				err = flapsClient.Wait(ctx, updateResult, "started")
-
 				if err != nil {
-					err = flapsClient.ReleaseLease(ctx, machine.ID, machine.LeaseNonce)
 					return err
 				}
 			}
-
-		}
-
-		for _, machine := range machines {
-
-			err = flapsClient.ReleaseLease(ctx, machine.ID, machine.LeaseNonce)
-			if err != nil {
-				fmt.Println(io.Out, fmt.Errorf("could not release lease on %s (%w)", machine.ID, err))
-			}
-
 		}
 
 	} else {
 		fmt.Fprintf(io.Out, "Launching VM with image %s\n", launchInput.Config.Image)
 		_, err = flapsClient.Launch(ctx, launchInput)
-
 		if err != nil {
 			return err
 		}
@@ -317,3 +296,10 @@ func DeployMachinesApp(ctx context.Context, app *api.AppCompact, strategy string
 
 	return
 }
+
+func releaseLease(ctx context.Context, client *flaps.Client, machine *api.Machine) {
+	io := iostreams.FromContext(ctx)
+	if err := client.ReleaseLease(ctx, machine.ID, machine.LeaseNonce); err != nil {
+		fmt.Println(io.Out, fmt.Errorf("could not release lease on %s (%w)", machine.ID, err))
+	}
+}
