@@ -4,11 +4,11 @@ import (
 	"context"
 	"fmt"
 
-	"github.com/pkg/errors"
 	"github.com/spf13/cobra"
 	"github.com/superfly/flyctl/agent"
 	"github.com/superfly/flyctl/api"
 	"github.com/superfly/flyctl/client"
+	"github.com/superfly/flyctl/flaps"
 	"github.com/superfly/flyctl/flypg"
 	"github.com/superfly/flyctl/internal/app"
 	"github.com/superfly/flyctl/internal/command"
@@ -57,23 +57,34 @@ func runDetach(ctx context.Context) error {
 		return fmt.Errorf("get app: %w", err)
 	}
 
+	agentclient, err := agent.Establish(ctx, client)
+	if err != nil {
+		return fmt.Errorf("can't establish agent %w", err)
+	}
+
+	dialer, err := agentclient.Dialer(ctx, app.Organization.Slug)
+	if err != nil {
+		return fmt.Errorf("can't build tunnel for %s: %s", app.Organization.Slug, err)
+	}
+	ctx = agent.DialerWithContext(ctx, dialer)
+
 	switch pgApp.PlatformVersion {
 	case "nomad":
 		if err := hasRequiredVersionOnNomad(pgApp, MinPostgresHaVersion, MinPostgresHaVersion); err != nil {
 			return err
 		}
 	case "machines":
-		agentclient, err := agent.Establish(ctx, client)
+
+		flapsClient, err := flaps.New(ctx, pgApp)
 		if err != nil {
-			return fmt.Errorf("can't establish agent %w", err)
+			return fmt.Errorf("list of machines could not be retrieved: %w", err)
 		}
 
-		dialer, err := agentclient.Dialer(ctx, app.Organization.Slug)
+		members, err := flapsClient.List(ctx, "started")
 		if err != nil {
-			return fmt.Errorf("can't build tunnel for %s: %s", app.Organization.Slug, err)
+			return fmt.Errorf("machines could not be retrieved %w", err)
 		}
-
-		leader, err := fetchLeader(ctx, pgApp, dialer)
+		leader, err := fetchPGLeader(ctx, members)
 		if err != nil {
 			return fmt.Errorf("can't fetch leader: %w", err)
 		}
@@ -108,16 +119,6 @@ func runDetach(ctx context.Context) error {
 
 	targetAttachment := attachments[selected]
 
-	agentclient, err := agent.Establish(ctx, client)
-	if err != nil {
-		return errors.Wrap(err, "can't establish agent")
-	}
-
-	dialer, err := agentclient.Dialer(ctx, pgApp.Organization.Slug)
-	if err != nil {
-		return fmt.Errorf("can't build tunnel for %s: %w", app.Organization.Slug, err)
-	}
-
 	pgclient := flypg.New(pgAppName, dialer)
 
 	// Remove user if exists
